/*
-----------------------------------------------------------------------------
This source file is part of OGRE
    (Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright (c) 2000-2005 The OGRE Team
Also see acknowledgements in Readme.html

You may use this sample code for anything you like, it is not covered by the
LGPL like the rest of the engine.
-----------------------------------------------------------------------------
*/

/**
    \file 
        Lighting.h
    \brief
        Shows lighting support in OGRE. Also demonstrates the use of billboards
        and controllers for automatic time-relative behaviour.
*/


#include "ExampleApplication.h"
#include "OgreIlluminationManager.h"
#include "SpriteParticleRenderer.h"


// Listener class for frame updates
class ParticleDemoFrameListener : public ExampleFrameListener
{
protected:
	
	unsigned long framecount;
	
public:
    ParticleDemoFrameListener(RenderWindow* window, Camera* maincam)
        :ExampleFrameListener(window, maincam) 
				 
    {
		framecount = 0;		
    }

    bool frameStarted(const FrameEvent& evt)
    {		
		bool result = ExampleFrameListener::frameStarted(evt);
       framecount++;
        return result;     
    }

	

};

/** Application class */
class ParticleDemoApplication : public ExampleApplication
{

public:
/*	virtual void go(void)
    {
        if (!setup())
            return;

        renderScene();
		
        // clean up
        destroyScene();
    }*/

protected:
	SceneNode* kupolaNode;
	SceneNode* mainLightNode;
	/*	
	void renderScene(){
		
		mWindow->resetStatistics();
		while(true){		

			
			#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32            
			// Pump events on Win32
    		MSG  msg;
			while( PeekMessage( &msg, NULL, 0U, 0U, PM_REMOVE ) )
			{
				TranslateMessage( &msg );
				DispatchMessage( &msg );
			}
			
			#endif
			
			if(!mRoot->_fireFrameStarted()){
				break;
			}
			unsigned long framenum = mRoot->getCurrentFrameNumber();
			
			OgreIlluminationManager::getSingleton().update(framenum, mWindow);
			
			//mRoot->renderOneFrame();
			mRoot->_updateAllRenderTargets();

			mRoot->_fireFrameEnded();	
			
		}
	}*/

	void loadResources(void)
	{
		// Initialise, parse scripts etc
		ResourceGroupManager::getSingleton().initialiseResourceGroup("Bootstrap");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("General");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("PostProc");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("GTP_Basic");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("GTP_EnvMap");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("GTP_Caustic");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("GTP_Particles");
	}

	void createPostproc()
	{		
		CompositorManager::getSingleton().addCompositor(mWindow->getViewport(0),
														"GTP/PostProc/Glow");
		CompositorManager::getSingleton().setCompositorEnabled(mWindow->getViewport(0),
														"GTP/PostProc/Glow", true);

		CompositorManager::getSingleton().addCompositor(mWindow->getViewport(0),
														"GTP/PostProc/ToneMap");
		CompositorManager::getSingleton().setCompositorEnabled(mWindow->getViewport(0),
														"GTP/PostProc/ToneMap", true);		
	}
    

	void createScene(void)
    {
		createPostproc();

		OgreIlluminationManager::getSingleton().setMainCamera(mCamera);
		OgreIlluminationManager::getSingleton().setMainViewport(mWindow->getViewport(0));
				
		Root::getSingleton()._setCurrentSceneManager(mSceneMgr);
		mCamera->setPosition(0,100,240);
		mCamera->setFOVy(Radian(Degree(80)));
		//mCamera->setFarClipDistance(1000);
		// Set ambient light
        mSceneMgr->setAmbientLight(ColourValue(1, 1, 1));

		// Skybox
        mSceneMgr->setSkyBox(true, "Examples/MorningSkyBox");
		SceneNode* rootNode = mSceneMgr->getRootSceneNode();
		
		
		ParticleSystem* pSys1 = mSceneMgr->createParticleSystem("tuzecske", "GTP/SBB");
		pSys1->setKeepParticlesInLocalSpace(false);
		SceneNode *pSysNode = rootNode->createChildSceneNode();
		pSysNode->attachObject(pSys1); 
		//pSysNode->setPosition(0,100,0);
			
      
		//init objects
		Entity* kupola = mSceneMgr->createEntity("kupola", "kupola.mesh");		
		Entity* head = mSceneMgr->createEntity("head", "ogrehead.mesh");
				
		//init nodes
		kupolaNode = rootNode->createChildSceneNode( "kupola" );
		kupolaNode->rotate(Vector3(0,1,0),Radian(Degree(90)));
		kupolaNode->attachObject( kupola );
		kupolaNode->scale(25,21,25);		
		kupolaNode->_updateBounds();
		MeshPtr kmesh = kupola->getMesh();
		kmesh->buildTangentVectors(0,1);
		
		SceneNode* headNode = rootNode->createChildSceneNode("head");
		headNode->attachObject(head);
		headNode->setScale(2,2,2);
		AxisAlignedBox bbox = head->getBoundingBox();
		headNode->setPosition(0, 100,0);		
		headNode->_updateBounds();
		

		//Init lights
		Light* mainLight = mSceneMgr->createLight("MainLight");
		mainLight->setType(Light::LT_POINT);
        mainLight->setDiffuseColour(ColourValue::White);
		mainLight->setSpecularColour(ColourValue::White);
		mainLightNode = rootNode->createChildSceneNode();
		mainLightNode->setPosition(0,150,0);
		mainLight->setPosition(mainLightNode->getWorldPosition());
		mainLightNode->attachObject(mainLight); 
		
		OgreIlluminationManager::getSingleton().initTechniques();
		/*
		{//SBB
				OgreSharedRuns* sharedruns = new OgreSharedRuns();
			OgreIlluminationManager::getSingleton().addSharedRuns(sharedruns);
			OgreTechniqueGroup* group = new OgreTechniqueGroup();
			group->addSharedRun(sharedruns);
			SpriteParticleRenderer* renderer = (SpriteParticleRenderer*)pSys1->getRenderer();
			OgreRenderable* rend = new OgreRenderable(renderer->getSpriteSet(), pSys1);			
			sharedruns->addRenderable(rend);
			sharedruns->updateBounds();		
			Material* mat = (Material*) MaterialManager::getSingleton().getByName("GameTools/SBB").getPointer();
		
			OgreSBBRenderTechnique* techn = 
			new OgreSBBRenderTechnique(1,
										mat->getTechnique(0)->getPass(0),
										rend,
										group
										);
			group->addRenderTechnique(techn);
			renderer->getSpriteSet()->setRenderTechniqueGroup(group);
		}*/
		
   }

	void createFrameListener(void)
    {
		// This is where we instantiate our own frame listener
		mFrameListener= new ParticleDemoFrameListener(mWindow,
											mCamera	);
		mFrameListener->setPriority(1);
		OgreIlluminationManager::getSingleton().setPriority(2);
    mRoot->addFrameListener(mFrameListener);
	mRoot->addFrameListener(&OgreIlluminationManager::getSingleton());
   }

};

